/*
Copyright 2021 J.Luis Álvarez.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

'use strict';

// ---------------
// --- Globals ---
// ---------------

var current_part = default_part;
const picture_size = 41 * 2;


// ----------------------
// --- Parts selector ---
// ----------------------

// Fill-up the parts div with the components above
// Each <img> tag must have the class "ComponentGrid"
function populate_parts() {
    //Clear the "parts" element
    let parts_div = document.getElementById("parts");
    while (parts_div.firstChild) {
        parts_div.removeChild(parts_div.firstChild);
    }

    //For each part, create a <span> block with an <img> inside pointing to the
    //adequate .svg picture
    for (let category in parts) {
        let category_parts = parts[category];
        let category_header = document.createElement("p");
        category_header.innerText = category;
        parts_div.appendChild(category_header);
        for (let key in category_parts) {
            let part = category_parts[key];
            let part_span = document.createElement('span');
            let part_img = document.createElement('img');

            //Make the IMG element
            part_img.setAttribute("src", part_to_img(part));
            part_img.setAttribute("class", "ComponentGrid");
            part_img.setAttribute("id", part);
            part_img.setAttribute("width", picture_size);
            part_img.setAttribute("height", picture_size);

            //Connect the IMG element to a click listener
            part_img.addEventListener("click", function () { on_part_click(this.id); });

            part_span.appendChild(part_img);
            parts_div.appendChild(part_span);
        }
    }

    //Set default part as current part
    set_current_part(default_part);
}

function set_current_part(id) {
    console.log("Current part <" + id + ">");
    let part_box = document.getElementById("current_part");

    //Make the IMG element
    let part_img = document.createElement('img');
    part_img.setAttribute("src", part_to_img(id));
    part_img.setAttribute("class", "ComponentGrid");
    part_img.setAttribute("width", picture_size);
    part_img.setAttribute("height", picture_size);

    //Set global var
    current_part = id;

    part_box.appendChild(part_img);
    part_box.removeChild(part_box.firstChild);
}

function on_part_click(id) {
    set_current_part(id);
}

// -------------------
// --- Kludge grid ---
// -------------------

const grid_w = 20;
const grid_h = 20;

function populate_grid() {
    //Clear the "parts" <table> element
    let grid = document.getElementById("grid");
    while (grid.firstChild) {
        grid.removeChild(grid.firstChild);
    }

    //Make a grid_w by grid_h table
    for (let y = 0; y < grid_h; y++) {
        let row = document.createElement('tr');
        row.setAttribute("class", "KludgeGrid");
        for (let x = 0; x < grid_w; x++) {
            //Make Table Data
            let cell = document.createElement('td');
            cell.setAttribute("class", "KludgeGrid");
            //Make IMG attribute
            let cell_img = document.createElement('img');
            cell_img.setAttribute("src", part_to_img(default_part));
            cell_img.setAttribute("id", "cell_" + ((y * grid_w) + x));
            cell_img.setAttribute("width", picture_size);
            cell_img.setAttribute("height", picture_size);
            //Connect listener
            cell_img.addEventListener("click", function () { on_cell_click(this.id); });

            cell.appendChild(cell_img);
            row.appendChild(cell);
        }
        grid.appendChild(row);
    }
}

function on_cell_click(id) {
    console.log("Setting cell <" + id + "> with part <" + current_part + ">");
    let cell_img = document.getElementById(id);
    cell_img.setAttribute("src", part_to_img(current_part));
}

console.log("[BEGIN] Populating parts")
populate_parts();
console.log("[ END ] Populating parts")

console.log("[BEGIN] Populating grid")
populate_grid();
console.log("[ END ] Populating grid")
